'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * `Client` makes it easy to send message to background
 * and enable to handle response with Promise.
 * @constructor
 * @param {object} module is expected to be like [chrome.runtime], which has `sendMessage` method.
 * @param {bool} strict specifies the response promise to be with `status: 200`, otherwise rejection of Promise would be thrown.
 * @param {string} method is a method name which should be invoked, belonging to `module`.
 */
var Client = exports.Client = function () {
    function Client(module) {
        var strict = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
        var method = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'sendMessage';

        _classCallCheck(this, Client);

        this.module = module;

        switch (typeof method === 'undefined' ? 'undefined' : _typeof(method)) {
            case 'string':
                this.method = method;
                this.strict = strict;
                break;
            case 'boolean':
                this.method = 'sendMessage';
                this.strict = method;
                break;
            default:
        }

        if (typeof this.module[this.method] != 'function') {
            throw 'this module doesn\'t have valid method with name "' + this.method + '"';
        }
    }

    /**
     * `connect` provides a client for long-lived connected port client.
     * @see https://developer.chrome.com/extensions/runtime#method-connect
     * @see https://developer.chrome.com/extensions/tabs#method-connect
     * @param {object} module is either `chrome.runtime` or `chrome.tabs`
     * @param {string} id     might be either `extensionId` or `tabId`
     * @param {object} info   might be `connectInfo`
     */


    _createClass(Client, [{
        key: 'message',


        /**
         * `message` is a shorthand to send message to (for example) background.
         * @param {string} name might be routing name to match (or to be resolved).
         * @param {object} params might be parameters to pass to controller.
         * @return {Promise} Result Promise
         */
        value: function message() {
            var _this = this;

            var args = new (Function.prototype.bind.apply(Array, [null].concat(Array.prototype.slice.call(arguments))))();
            // If the first argument is string, it might represent "action".
            // Ensure the first argument to be an Object.
            if (typeof args[0] == 'string') {
                if (_typeof(args[1]) == 'object') {
                    args[1].action = args[0];
                    args = args.slice(1);
                } else {
                    args[0] = { action: args[0] };
                }
            }
            return new Promise(function (resolve, reject) {
                var _module$method;

                (_module$method = _this.module[_this.method]).call.apply(_module$method, [_this.module].concat(_toConsumableArray(_this._expandArgumentsForChromeMethod(args, resolve, reject))));
            });
        }

        /**
         * `_expandArgumentsForChromeMethod` expands given arguments to `message` method,
         * so that the original method of (for example) `chrome.runtime.sendMessage`
         * can handle that arguments.
         * @param {array} args arguments give to `message` method.
         * @param {function} resolve to resolve Response Promise.
         * @param {function} reject to reject Response Promise.
         */

    }, {
        key: '_expandArgumentsForChromeMethod',
        value: function _expandArgumentsForChromeMethod(args, resolve, reject) {
            var _this2 = this;

            var strict = this._isStrictMode(args);

            // If the last argument of `message` is function, respect it and invoke.
            if (typeof args[args.length - 1] == 'function') {
                // Replace the last argument as Callback for chrome interface.
                var cb = args[args.length - 1];
                args[args.length - 1] = function (responseFromBackend) {
                    cb(responseFromBackend);
                    resolve(); // resolve anyway
                    return true;
                };
            } else {
                // Append callback to satisfy chrome interface.
                var _cb = function _cb(responseFromBackend) {
                    _this2._finally(strict, responseFromBackend, resolve, reject);
                    return true;
                };
                args.push(_cb);
            }
            return args;
        }

        /**
         * `_isStrictMode`
         * @private
         * @param {array} givenArgs
         * @return {bool}
         */

    }, {
        key: '_isStrictMode',
        value: function _isStrictMode(givenArgs) {
            if (typeof givenArgs[givenArgs.length - 1] == 'boolean') {
                return givenArgs[givenArgs.length - 1];
            }
            return this.strict;
        }

        /**
         * `_finally` just dispatch resolve / reject conditions.
         */

    }, {
        key: '_finally',
        value: function _finally(strict, response, resolve, reject) {
            var status = parseInt(response.status);
            if (strict && (status < 200 || 400 <= status)) {
                reject(response);
            } else {
                resolve(response);
            }
        }

        /**
         * tab
         */

    }, {
        key: 'tab',
        value: function tab(tabId) {
            var strict = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
            var method = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'sendMessage';

            return new TabClient(tabId, this.module, strict, method);
        }
        /**
         * @static for
         * This is just a shorthand of `(new Client(chrome.tabs)).tab(123) // == TabClient`
         * This is kind of insignificant and useless method.
         * Just want to chain constructor and `tab`,
         * like this `Client.for(chrome.tabs, 123).message()`
         */

    }], [{
        key: 'connect',
        value: function connect(module) {
            var id = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
            var info = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

            var port = module.connect(id, info);
            return new this(port, true, 'postMessage');
        }
    }, {
        key: 'for',
        value: function _for(module, id) {
            var strict = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
            var method = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 'sendMessage';

            return new TabClient(id, module, strict, method);
        }
    }]);

    return Client;
}();

/**
 * TabClient converts given arguments to arguments for chrome.tabs.sendMessage
 * because it requires `tabId` for the first argument.
 */


var TabClient = function (_Client) {
    _inherits(TabClient, _Client);

    function TabClient(tabId, module) {
        var strict = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
        var method = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 'sendMessage';

        _classCallCheck(this, TabClient);

        var _this3 = _possibleConstructorReturn(this, (TabClient.__proto__ || Object.getPrototypeOf(TabClient)).call(this, module, strict, method));

        _this3.tabId = tabId;
        return _this3;
    }
    /**
     * @override
     */


    _createClass(TabClient, [{
        key: '_expandArgumentsForChromeMethod',
        value: function _expandArgumentsForChromeMethod(args, resolve, reject) {
            args.unshift(this.tabId);
            return _get(TabClient.prototype.__proto__ || Object.getPrototypeOf(TabClient.prototype), '_expandArgumentsForChromeMethod', this).call(this, args, resolve, reject);
        }
    }]);

    return TabClient;
}(Client);