'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

// Inspired by React.PropTypes
// https://github.com/facebook/react/blob/bc2702f8bd89737898874e05adc67a4b4f542929/src/isomorphic/classic/types/ReactPropTypes.js#L157
var createTypeChecker = function createTypeChecker(typename, validate) {
    var checkType = function checkType(required, value, name) {
        if (typeof value === 'undefined') {
            if (required) throw name + ' is marked as required';else return null;
        }
        if (!validate(value)) throw name + ' is not ' + typename;
        return null;
    };
    var checker = checkType.bind(null, false);
    checker.isRequired = checkType.bind(null, true);
    return checker;
};

var _createRecursiveTypeChecker = function _createRecursiveTypeChecker(structName, iterateNames) {
    // e.g. shape
    var generate = function generate() {
        var validations = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

        var checkRoot = function checkRoot(required, rootValue, rootName) {
            if (required && typeof rootValue == 'undefined') throw rootName + ' is marked as required';
            iterateNames(validations).map(function (fieldName) {
                var validation = validations[fieldName];
                var value = rootValue[fieldName];
                validation(value, fieldName);
            });
        };
        var check = checkRoot.bind(null, false);
        check.isRequired = checkRoot.bind(null, true);
        return check;
    };
    // let generator = generate.bind(null, false);
    // generator.isRequired = generate.bind(null, true);
    return generate;
};

var Types = {
    string: createTypeChecker('string', function (value) {
        return typeof value == 'string';
    }),
    number: createTypeChecker('number', function (value) {
        return typeof value == 'number';
    }),
    object: createTypeChecker('object', function (value) {
        return (typeof value === 'undefined' ? 'undefined' : _typeof(value)) == 'object';
    }),
    bool: createTypeChecker('bool', function (value) {
        return typeof value == 'boolean';
    }),
    array: createTypeChecker('array', function (value) {
        return Array.isArray(value);
    }),
    // localStorage cannot store function ;)
    // func:   createTypeChecker('function', value => typeof value == 'function'),
    shape: _createRecursiveTypeChecker('shape', function (value) {
        return Object.keys(value);
    })
};

exports.default = Types;