'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.Model = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _Types = require('./Types');

var _Types2 = _interopRequireDefault(_Types);

var _OnMemoryStorage = require('./OnMemoryStorage');

var _OnMemoryStorage2 = _interopRequireDefault(_OnMemoryStorage);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Model = exports.Model = function () {
    _createClass(Model, null, [{
        key: 'useStorage',
        value: function useStorage() {
            var storage = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

            if (typeof storage.getItem !== 'function') {
                throw '`getItem` of Storage interface is missing';
            }
            if (typeof storage.setItem !== 'function') {
                throw '`setItem` of Storage interface is missing';
            }
            if (typeof storage.removeItem !== 'function') {
                throw 'it doesn\'t satisfy expected Storage interface';
            }
            if (storage.getItem.length < 1) {
                throw '`getItem` of Storage must accept at least 1 argument';
            }
            if (storage.setItem.length < 2) {
                throw '`setItem` of Storage must accept at least 2 argument';
            }
            if (storage.removeItem.length < 1) {
                throw '`removeItem` of Storage must accept at least 1 argument';
            }
            this.__storage = storage;
        }
    }]);

    function Model() {
        var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
        var id = arguments[1];
        var ns = arguments[2];

        _classCallCheck(this, Model);

        this._props = props || {};
        this._id = id;
        this._ns = ns || this.constructor.name;
        this.decode(props);
    }

    _createClass(Model, [{
        key: 'save',
        value: function save() {
            var all = this.constructor._all();
            if (!this._id) this._id = this.constructor.__nextID(all);
            this._validate(); // TODO: should be response object??
            all[this._id] = this.encode();
            this.constructor.__storage.setItem(this.constructor.name, JSON.stringify(all));
            return this;
        }
    }, {
        key: 'delete',
        value: function _delete() {
            var all = this.constructor._all();
            if (!this._id) return false;
            if (delete all[this._id] === false) return false;
            this.constructor.__storage.setItem(this.constructor.name, JSON.stringify(all));
            return true;
        }
    }, {
        key: 'update',
        value: function update(dict) {
            var _this2 = this;

            if ((typeof dict === 'undefined' ? 'undefined' : _typeof(dict)) != 'object') return this.error('Argument for `update` must be key-value dictionary');
            // TODO: filter preserved keywords
            Object.keys(dict).map(function (key) {
                return _this2[key] = dict[key];
            });
            return !!this.save();
        }
    }, {
        key: 'error',
        value: function error(err) {
            this.errors = this.errors || [];
            this.errors.push(err);
            return false;
        }
    }, {
        key: 'decode',
        value: function decode(obj) {
            var _this3 = this;

            Object.keys(obj).map(function (key) {
                _this3[key] = obj[key];
            });
            return this;
        }
    }, {
        key: 'encode',
        value: function encode() {
            var _this4 = this;

            var encoded = {};

            var _loop = function _loop(prop) {
                if (['_id', '_ns', '_props'].some(function (x) {
                    return x == prop;
                })) return 'continue';
                if (_this4.hasOwnProperty(prop)) encoded[prop] = _this4[prop];
            };

            for (var prop in this) {
                var _ret = _loop(prop);

                if (_ret === 'continue') continue;
            }
            return encoded;
        }
    }, {
        key: '_validate',
        value: function _validate() {
            var _this5 = this;

            if (!this.constructor.schema) return null;
            Object.keys(this.constructor.schema).map(function (propName) {
                _this5.constructor.schema[propName](_this5[propName], propName);
            });
        }
    }], [{
        key: 'new',
        value: function _new() {
            var template = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : Object.assign({}, this.template || {});

            var props = {};
            Object.keys(template).map(function (key) {
                props[key] = typeof template[key] == 'function' ? template[key]() : template[key];
            });
            return new this(props);
        }

        /**
         * it returns all saved entities **as Objects**
         */

    }, {
        key: '_all',
        value: function _all() {
            var _ns = this.name;
            var raw = this.__storage.getItem(_ns);
            var all = JSON.parse(raw || 'null');
            return all || this.default || {};
        }
        /**
         * it returs all saved entities **as Models**
         */

    }, {
        key: 'all',
        value: function all() {
            var _this6 = this;

            var all = this._all();
            Object.keys(all).map(function (_id) {
                var _this = new _this6(all[_id], _id, _this6.name);
                all[_id] = _this.decode(all[_id]);
            });
            return all;
        }

        /**
         * list is an alias of `Model.filter(() => true);`
         * @return {array} of model instances
         */

    }, {
        key: 'list',
        value: function list() {
            return this.filter();
        }
    }, {
        key: 'drop',
        value: function drop() {
            this.__storage.removeItem(this.name);
        }
    }, {
        key: 'create',
        value: function create() {
            var dict = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.template || {};

            if ((typeof dict === 'undefined' ? 'undefined' : _typeof(dict)) != 'object') return;
            var all = this._all();
            var _id = dict._id || this.__nextID(all);
            var model = new this(dict, _id);
            return model.save();
        }
    }, {
        key: 'find',
        value: function find(id) {
            var all = this._all();
            var _id = String(id);
            if (all[_id]) {
                var _this = new this(all[_id], _id, this.name);
                return _this.decode(all[_id]);
            } else if (this.default && this.default[_id]) {
                var _this7 = new this(this.default[_id], _id, this.name);
                return _this7.decode(this.default[_id]);
            }
        }
    }, {
        key: 'filter',
        value: function filter() {
            var fn = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : function () {
                return true;
            };

            var all = this.all();
            var _res = [];
            Object.keys(all).map(function (_id) {
                if (fn(all[_id])) _res.push(all[_id]);
            });
            return _res;
        }
    }, {
        key: '__nextID',
        value: function __nextID(all) {
            if (typeof this.nextID === 'function') {
                var id = this.nextID(all);
                if (id === undefined || id === null) {
                    console.error('customized `nextID` function provides invalid ID');
                    return this.timestampID;
                }
                return id;
            }
            return this.timestampID();
        }
    }, {
        key: 'nextID',
        value: function nextID() /* all */{
            return this.timestampID(); // for default
        }
    }, {
        key: 'timestampID',
        value: function timestampID() {
            return String(Date.now());
        }
    }, {
        key: 'sequentialID',
        value: function sequentialID(all) {
            return (Object.keys(all).map(function (id) {
                return parseInt(id);
            }).sort(function (prev, next) {
                return prev < next ? -1 : 1;
            }).pop() || 0) + 1;
        }
    }]);

    return Model;
}();

Model.Types = _Types2.default;
Model.__storage = global.localStorage ? global.localStorage : new _OnMemoryStorage2.default();