# chomex.Model

`chomex.Model` is just an accessor of `localStorage`, behaves like ActiveRecord.

- Methods
  - [new](#new)
  - [save](#save)
  - [find](#find)
  - [update](#update)
  - [delete](#delete)
  - [create](#create)
  - [all](#all)
  - [list](#list)
  - [filter](#filter)
- Properties
  - [schema](#schema)
  - [nextID](#nextID)

# How to use

```js
class Player extends Model {}
let player = new Player({name: 'otiai10', age: 31});
player.name // 'otiai10'
player.age // 31
player._id // undefined, because not saved yet

player.save();
player._id // 1, because it's saved to localStorage
```

# Methods

## new

- static
- an alias for `constructor`

```js
let player = Player.new({name: 'otiai20', age: 43});
player.name // 'otiai10'
player.age // 31
player._id // undefined, because not saved yet
```

## save

```js
let player = new Player({name: 'otiai20'});
player.save();
player._id // 2
```

## find

- static

```js
let player = Player.find(2);
player.name // 'otiai10'
```

## update

```js
player.update({name: 'otiai22'});
Player.find(player._id).name // 'otiai22'
```

## delete

```js
player.delete();
Player.find(player._id) // undefined
```

## create

- static
- an alias for `new` and `save`

```js
let player = Player.create({name: 'otiai99', age: 99});
player._id // 3

// Is equivalent to
Player.new({name: 'otiai99'}).save();
```

## all

- static
- returns everything as a dictionary

```js
const dict = Player.all(); // Object
dict[1].name // 'otiai10'
dict[1] instanceof Player // true
```

## list

- static
- returns everything as an array

```js
const players = Player.list(); // [Player]
players.length // 2

// Is equivalent to
Player.filter(() => true);
```

## filter

- static
- returns filtered array by filterFunc

```js
const players = Player.filter(p => p.age < 40);
players.length // 1
```

## useStorage

- static
- replace storage with anything which satisfies Storage interface

```js
Model.useStorage(window.sessionStorage);

// For example, you can embed any extra operation for getItem/setItem/removeItem
const storage = new MyStorageWithEffortAsyncPushing();
Model.useStorage(storage);
```

# Properties

## schema

- static
- optional, default `undefined`
- can define validations for each props of this model
- no validations, if `schema` is not set

```js
class Player extends Model {
  static schema = {
    name: Model.Types.string.isRequired,
    age:  Model.Types.number, // optional
    location: Model.Types.shape({
      address: Model.Types.string,
      visible: Model.Types.bool.isRequired,
    }),
  }
}
```

## nextID

- static
- optional, default `timestampID`
- replace it if you want to change algorythm of generating next id

```js
Player.nextID = () => Date.now();
Player.create({name: 'otiai2017'})._id // 1488061388247
Player.create({name: 'otiai1986'})._id // 1488061388928
```
